﻿using Kivii;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Timeliness.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Jst.Lims.Timeliness
{
    [Api(Description = "时效统计")]
    public class TimelineStatistic
    {
        public Guid Kvid { get; set; }

        public Guid RootKvid { get; set; }

        public Guid ParentKvid { get; set; }

        public string OperatorName { get; set; }

        public string OwnerName { get; set; }

        public Guid OwnerKvid { get; set; }//当前节点所属部门

        public string ReportId { get; set; }//报告信息

        public string Title { get; set; }//节点信息
        
        public string Summary { get; set; }

        public string Remark { get; set; }

        public TimeFilter TimeFilter { get; set; }//时间筛选

        public decimal Quantity { get; set; }
        public decimal TimeoutQuantity { get; set; }
        public decimal NormalQuantity { get; set; }
        public decimal TimeFilterQuantity { get; set; }

        public DateTime? PlanStartTime { get; set; }
        public DateTime? RealStartTime { get; set; }
        public DateTime? PlanFinishTime { get; set; }
        public DateTime? RealFinishTime { get; set; }
        public DateTime? ReportDealDate { get; set; }
        public DateTime? ReportDeadDate { get; set; }
        public DateTime? ReportPreparationDate { get; set; }//编制时间
        public DateTime? ReportReviewDate { get; set; }//审核时间
        public DateTime? ReportIssueDate { get; set; }//签发时间

        public decimal? ReportUrgentRate { get; set; }

        public List<TimelineStatistic> Details { get; set; }

    }

    [Api(Description = "报告数据录入/报告编制时效统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class TimelineStatisticDataEntry : RestfulExecution<TimelineStatistic>
    {
        public Guid PolicyNodeKvid { get; set; }

        /// <summary>
        /// 统计的开始日期
        /// </summary>
        public DateTime BeginDate { get; set; }

        /// <summary>
        /// 统计的结束日期
        /// </summary>
        public DateTime EndDate { get; set; }

        public TimeFilter TimeFilter { get; set; }

        public bool Grouped { get; set; }
        public bool? Urgent { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginDate == DateTime.MinValue) BeginDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndDate == DateTime.MinValue) EndDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginDate = DateTime.Parse(BeginDate.ToString("yyyy-MM-dd"));
            EndDate = DateTime.Parse(EndDate.ToString("yyyy-MM-dd"));
            if (EndDate < BeginDate) throw new Exception("查询结束日期不可小于开始日期!");

            var rtns = new RestfulQueryResponse<TimelineStatistic>();
            rtns.Results = new List<TimelineStatistic>();

            var connL = KiviiContext.GetOpenedDbConnection<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Timeline>();

            var results = new List<TimelineStatistic>();
            if (connL.ConnectionString == conn.ConnectionString)//同一个数据库下的情况 用leftjoin联合查询
            {
                //connL.Close();
                connL = conn;

                var query = conn.From<Timeline>();
                query.LeftJoin<Report>((o, r) => o.BizKvid == r.Kvid);
                if (PolicyNodeKvid == Guid.Empty) query.Where<Timeline>(o => o.PreKvid == Guid.Empty && o.RootKvid == o.Kvid);//不指定节点，就查根节点
                else query.Where<Timeline>(o => o.PolicyNodeKvid == PolicyNodeKvid && o.PreKvid != Guid.Empty);//指定节点，不能包含根节点
                query.Where<Report>(o => o.DeadDate >= BeginDate && o.DeadDate <= EndDate && Sql.In(o.TypeEx, "W", "Z"));//只查看不加急并且要求完成日期在所选范围内的时效记录
                query.And<Report>(o => !o.ReportId.ToLower().Contains("test"));
                if (Urgent != null)
                {
                    if (Urgent.Value) query.And<Report>(o => o.UrgentRate > 1);
                    else query.And<Report>(o => o.UrgentRate == 1);
                }
                query.And<Report>(o => (o.BizType != "G0" && o.BizType != "G1") || o.BizType == null);
                query.OrderBy<Report>(o => o.DeadDate);
                query.Select<Timeline, Report>((o, r) => new
                {
                    o.Kvid,
                    o.RootKvid,
                    o.ParentKvid,
                    o.OperatorName,
                    o.OwnerKvid,
                    o.OwnerName,
                    r.ReportId,
                    o.Title,
                    o.PlanStartTime,
                    o.RealStartTime,
                    o.PlanFinishTime,
                    o.RealFinishTime,
                    o.Summary,
                    o.Remark,
                    ReportDealDate = r.DealDate,
                    ReportDeadDate = r.DeadDate,
                    ReportPreparationDate = r.PreparationDate,
                    ReportReviewDate = r.ReviewDate,
                    ReportIssueDate = r.IssueDate,
                    ReportUrgentRate = r.UrgentRate

                });
                results = conn.Select<TimelineStatistic>(query);

            }
            if (Grouped)
            {
                if (PolicyNodeKvid == Guid.Empty)//不按照节点Group统计，按照整个报告的时效进行统计
                {
                    var group = results.GroupBy(o => o.ReportDeadDate);
                    foreach (var kv in group)
                    {
                        DateTime? filterTime = null;
                        switch (TimeFilter)
                        {
                            case TimeFilter.Unsupported:
                                filterTime = null;
                                break;
                            case TimeFilter.TwoPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 14:00:00"));
                                break;
                            case TimeFilter.ThreePM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 15:00:00"));
                                break;
                            case TimeFilter.FourPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:00:00"));
                                break;
                            case TimeFilter.FourHalfPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 16:30:00"));
                                break;
                            case TimeFilter.FiveHalfPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 17:30:00"));
                                break;
                            case TimeFilter.SixPM:
                                filterTime = DateTime.Parse(kv.Key.Value.ToString("yyyy-MM-dd 18:00:00"));
                                break;
                            default:
                                filterTime = null;
                                break;
                        }
                        var total = new TimelineStatistic();
                        total.ReportDeadDate = kv.Key;
                        total.Quantity = kv.Count();
                        total.TimeFilter = TimeFilter;
                        total.Details = new List<TimelineStatistic>();
                        foreach (var item in kv)
                        {
                            var realFinishTime = item.RealFinishTime;
                            if (realFinishTime == null) realFinishTime = DateTime.Now;
                            if (item.PlanFinishTime != item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1);
                            if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimeoutQuantity++;
                            else total.NormalQuantity++;
                            if (filterTime != null && item.RealFinishTime != null && item.RealFinishTime <= filterTime) total.TimeFilterQuantity++;
                            else item.TimeoutQuantity = 1;
                        }
                        total.Details = kv.ToList();
                        rtns.Results.Add(total);
                    }
                }
                else//按照节点Group
                {
                    var group = results.GroupBy(o => new { o.ReportDeadDate, o.Title });
                    foreach (var kv in group)
                    {

                        DateTime? filterTime = null;
                        switch (TimeFilter)
                        {
                            case TimeFilter.Unsupported:
                                filterTime = null;
                                break;
                            case TimeFilter.TwoPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 14:00:00"));
                                break;
                            case TimeFilter.ThreePM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 15:00:00"));
                                break;
                            case TimeFilter.FourPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 16:00:00"));
                                break;
                            case TimeFilter.FourHalfPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 16:30:00"));
                                break;
                            case TimeFilter.FiveHalfPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 17:30:00"));
                                break;
                            case TimeFilter.SixPM:
                                filterTime = DateTime.Parse(kv.Key.ReportDeadDate.Value.ToString("yyyy-MM-dd 18:00:00"));
                                break;
                            default:
                                filterTime = null;
                                break;
                        }
                        var total = new TimelineStatistic();
                        total.ReportDeadDate = kv.Key.ReportDeadDate;
                        total.Title = kv.Key.Title;
                        total.Quantity = kv.Count();
                        total.TimeFilter = TimeFilter;
                        total.Details = new List<TimelineStatistic>();
                        foreach (var item in kv)
                        {
                            var realFinishTime = item.RealFinishTime;
                            if (realFinishTime == null) realFinishTime = DateTime.Now;
                            //if (item.PlanFinishTime != item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1)) item.PlanFinishTime = item.ReportDeadDate.Value.AddDays(1).AddSeconds(-1);
                            if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime) total.TimeoutQuantity++;
                            else total.NormalQuantity++;
                            if (filterTime != null && item.RealFinishTime != null && item.RealFinishTime <= filterTime) total.TimeFilterQuantity++;
                            else item.TimeoutQuantity = 1;
                        }
                        total.Details = kv.ToList();
                        rtns.Results.Add(total);
                    }
                }
            }
            else rtns.Results = results;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "报告委托受理/开样时效统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class TimelineStatisticCommission : RestfulExecution<TimelineStatistic>
    {
        public Guid PolicyNodeKvid { get; set; }

        /// <summary>
        /// 统计的开始日期
        /// </summary>
        public DateTime BeginDate { get; set; }

        /// <summary>
        /// 统计的结束日期
        /// </summary>
        public DateTime EndDate { get; set; }

        //public TimeFilter TimeFilter { get; set; }

        public bool Grouped { get; set; }
        public bool? Urgent { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginDate == DateTime.MinValue) BeginDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndDate == DateTime.MinValue) EndDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginDate = DateTime.Parse(BeginDate.ToString("yyyy-MM-dd"));
            EndDate = DateTime.Parse(EndDate.ToString("yyyy-MM-dd"));
            if (EndDate < BeginDate) throw new Exception("查询结束日期不可小于开始日期!");

            var rtns = new RestfulQueryResponse<TimelineStatistic>();
            rtns.Results = new List<TimelineStatistic>();

            var connL = KiviiContext.GetOpenedDbConnection<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Timeline>();

            var results = new List<TimelineStatistic>();
            if (connL.ConnectionString == conn.ConnectionString)//同一个数据库下的情况 用leftjoin联合查询
            {
                //connL.Close();
                connL = conn;

                var query = conn.From<Timeline>();
                query.LeftJoin<Report>((o, r) => o.BizKvid == r.Kvid);
                if (PolicyNodeKvid == Guid.Empty) query.Where<Timeline>(o => o.PreKvid == Guid.Empty && o.RootKvid == o.Kvid);//不指定节点，就查根节点
                else query.Where<Timeline>(o => o.PolicyNodeKvid == PolicyNodeKvid && o.PreKvid != Guid.Empty);//指定节点，不能包含根节点
                query.Where<Report>(o => o.DealDate >= BeginDate && o.DealDate <= EndDate && Sql.In(o.TypeEx, "W", "Z"));//只查看不加急并且要求完成日期在所选范围内的时效记录
                query.And<Report>(o => !o.ReportId.ToLower().Contains("test"));
                if (Urgent != null)
                {
                    if (Urgent.Value) query.And<Report>(o => o.UrgentRate > 1);
                    else query.And<Report>(o => o.UrgentRate == 1);
                }
                query.And<Report>(o => (o.BizType != "G0" && o.BizType != "G1") || o.BizType == null);
                query.OrderBy<Report>(o => o.DealDate);
                query.Select<Timeline, Report>((o, r) => new
                {
                    o.Kvid,
                    o.RootKvid,
                    o.ParentKvid,
                    o.OwnerKvid,
                    o.OwnerName,
                    o.OperatorName,
                    r.ReportId,
                    o.Title,
                    o.PlanStartTime,
                    o.RealStartTime,
                    o.PlanFinishTime,
                    o.RealFinishTime,
                    o.Summary,
                    o.Remark,
                    ReportDealDate = r.DealDate,
                    ReportDeadDate = r.DeadDate,
                    ReportPreparationDate = r.PreparationDate,
                    ReportReviewDate = r.ReviewDate,
                    ReportIssueDate = r.IssueDate,
                    ReportUrgentRate = r.UrgentRate

                });
                results = conn.Select<TimelineStatistic>(query);

            }
            if (Grouped)
            {
                if (PolicyNodeKvid == Guid.Empty)//不按照节点Group统计，按照整个报告的时效进行统计
                {
                    var group = results.GroupBy(o => o.ReportDealDate);
                    foreach (var kv in group)
                    {
                        var total = new TimelineStatistic();
                        total.ReportDealDate = kv.Key;
                        total.Quantity = kv.Count();
                        total.Details = new List<TimelineStatistic>();
                        var records = kv.OrderBy(o => o.PlanFinishTime).ToList();
                        var grop = records.GroupBy(o => o.PlanFinishTime);
                        foreach (var gp in grop)
                        {
                            var detail = new TimelineStatistic();
                            detail.PlanFinishTime = gp.Key;
                            detail.Quantity = gp.Count();
                            detail.Details = new List<TimelineStatistic>();
                            foreach (var item in gp)
                            {
                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime)
                                {
                                    detail.TimeoutQuantity++;
                                    item.TimeoutQuantity++;
                                    total.TimeoutQuantity++;
                                }
                                else
                                {
                                    detail.NormalQuantity++;
                                    item.NormalQuantity++;
                                    total.NormalQuantity++;
                                }
                                detail.Details.Add(item);
                            }
                            total.Details.Add(detail);
                        }
                        rtns.Results.Add(total);
                    }
                }
                else//按照节点Group
                {
                    var group = results.GroupBy(o => new { o.ReportDealDate, o.Title });
                    foreach (var kv in group)
                    {
                        var total = new TimelineStatistic();
                        total.ReportDealDate = kv.Key.ReportDealDate;
                        total.Title = kv.Key.Title;
                        total.Quantity = kv.Count();
                        total.Details = new List<TimelineStatistic>();
                        var records = kv.OrderBy(o => o.PlanFinishTime).ToList();
                        var grop = records.GroupBy(o => o.PlanFinishTime);
                        foreach (var gp in grop)
                        {
                            var detail = new TimelineStatistic();
                            detail.PlanFinishTime = gp.Key;
                            detail.Quantity = gp.Count();
                            detail.Details = new List<TimelineStatistic>();
                            foreach (var item in gp)
                            {
                                var realFinishTime = item.RealFinishTime;
                                if (realFinishTime == null) realFinishTime = DateTime.Now;
                                if (item.PlanFinishTime != null && realFinishTime > item.PlanFinishTime)
                                {
                                    detail.TimeoutQuantity++;
                                    item.TimeoutQuantity++;
                                    total.TimeoutQuantity++;
                                }
                                else
                                {
                                    detail.NormalQuantity++;
                                    item.NormalQuantity++;
                                    total.NormalQuantity++;
                                }
                                detail.Details.Add(item);
                            }
                            total.Details.Add(detail);
                        }
                        rtns.Results.Add(total);
                    }
                }
            }
            else rtns.Results = results;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    public enum TimeFilter
    {
        Unsupported = 0,

        /// <summary>
        /// 14:00
        /// </summary>
        TwoPM,

        /// <summary>
        /// 15:00
        /// </summary>
        ThreePM,

        /// <summary>
        /// 16:00
        /// </summary>
        FourPM,

        /// <summary>
        /// 16:30
        /// </summary>
        FourHalfPM,

        /// <summary>
        /// 17:30
        /// </summary>
        FiveHalfPM,

        /// <summary>
        /// 18:00
        /// </summary>
        SixPM
    }
}
